# Copyright (C) 2010-2015 Cuckoo Foundation.
# This file is part of Cuckoo Sandbox - http://www.cuckoosandbox.org
# See the file 'docs/LICENSE' for copying permission.

from nose.tools import assert_true, assert_false, assert_equal
import email
import tempfile

from lib.cuckoo.common.email_utils import find_attachments_in_email

class TestEmailUtils:
    def test_find_attachments(self):
        atts = find_attachments_in_email(SAMPLE_EMAIL, False)
        assert_equal(len(atts), 1)
        assert_false(any(t[1] == u'\u60e1\u610f\u8edf\u9ad4.doc' for t in atts))
        assert_true(any(t[1] == u'<unknown>.doc' for t in atts))

    def test_find_attachments_expand(self):
        atts = find_attachments_in_email(SAMPLE_EMAIL, True)
        assert_equal(len(atts), 3)
        assert_true(any(t[1] == u'\u60e1\u610f\u8edf\u9ad4.doc' for t in atts))

SAMPLE_EMAIL = """

MIME-Version: 1.0
Date: Sat, 6 Jun 2015 16:23:40 -0700
Delivered-To: example@example.com
Subject: test
From: Example <example@example.com>
To: Example <example@example.com>
Content-Type: multipart/mixed; boundary=001a113df348f253900517e1b282

--001a113df348f253900517e1b282
Content-Type: multipart/alternative; boundary=001a113df348f253550517e1b280

--001a113df348f253550517e1b280
Content-Type: text/plain; charset=UTF-8

test

--001a113df348f253550517e1b280
Content-Type: text/html; charset=UTF-8

<div dir="ltr">test
</div>

--001a113df348f253550517e1b280--
--001a113df348f253900517e1b282
Content-Type: text/plain; charset=US-ASCII; name="multipart.eml"
Content-Disposition: attachment; filename="multipart.eml"
Content-Transfer-Encoding: base64
X-Attachment-Id: f_ialo2rvt0
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--001a113df348f253900517e1b282
Content-Type: application/msword
Content-Disposition: attachment
Content-Transfer-Encoding: base64
X-Attachment-Id: f_ialo2ytw1

5oOh5oSP6Luf6auU
--001a113df348f253900517e1b282--


"""
